@@ -101,6 +101,7 @@ DLLEXPORT void gks_cairoplugin(int fctid, int dx, int dy, int dimx, int *i_arr,
 #define MAX_TNR 9
 
 #define HEIGHT_IN_CELLS 24
+#define CHUNK_SIZE 4096
 
 #define STR(x) #x
 #define XSTR(x) STR(x)
@@ -211,6 +212,93 @@ static int predef_ints[] = {0, 1, 3, 3, 3};
 
 static int predef_styli[] = {1, 1, 1, 2, 3};
 
+#ifndef _WIN32
+static enum tmux_state_t have_tmux(void)
+{
+  const char *term_env_var = gks_getenv("TERM");
+  if (term_env_var == NULL)
+    {
+      return NO_TMUX;
+    }
+
+  if (strncmp(term_env_var, "screen", 6) == 0 || strncmp(term_env_var, "tmux", 4) == 0)
+    {
+      /* Check if the tmux session is running locally, otherwise the server cannot be queried */
+      if (gks_getenv("TMUX") != NULL)
+        {
+          FILE *fp;
+          char client_termname[80];
+
+          /* Inside tmux we can query the tmux server for the outer terminal name */
+          fp = popen("tmux display -p '#{client_termname}'", "r");
+          if (fp == NULL)
+            {
+              /* Reading failed, assume a single tmux session */
+              return SINGLE_TMUX_SESSION;
+            }
+          /* Read the output a line at a time - output it. */
+          if (fgets(client_termname, sizeof(client_termname), fp) == NULL)
+            {
+              /* Reading failed, assume a single tmux session */
+              pclose(fp);
+              return SINGLE_TMUX_SESSION;
+            }
+          pclose(fp);
+          return (strncmp(client_termname, "screen", 6) == 0 || strncmp(client_termname, "tmux", 4) == 0)
+                     ? NESTED_TMUX_SESSION
+                     : SINGLE_TMUX_SESSION;
+        }
+      return SINGLE_TMUX_SESSION;
+    }
+
+  return NO_TMUX;
+}
+
+static void send_tmux_escape_sequence_start(void)
+{
+  if (!p->have_tmux) return;
+
+  if (p->have_tmux == SINGLE_TMUX_SESSION)
+    {
+      fprintf(stdout, "\033Ptmux;"); /* Start a tmux pass-through sequence */
+    }
+  else
+    {
+      fprintf(stdout, "\033Ptmux;\033\033Ptmux;"); /* Start a nested tmux pass-through sequence */
+    }
+}
+
+static void send_tmux_escaped_escape(void)
+{
+  switch (p->have_tmux)
+    {
+    case NESTED_TMUX_SESSION:
+      fprintf(stdout, "\033\033\033\033");
+      break;
+    case SINGLE_TMUX_SESSION:
+      fprintf(stdout, "\033\033");
+      break;
+    default:
+      fprintf(stdout, "\033");
+      break;
+    }
+}
+
+static void send_tmux_escape_sequence_end(void)
+{
+  if (!p->have_tmux) return;
+
+  if (p->have_tmux == SINGLE_TMUX_SESSION)
+    {
+      fprintf(stdout, "\033\\"); /* End a tmux pass-through sequence */
+    }
+  else
+    {
+      fprintf(stdout, "\033\033\\\033\\"); /* End a nested tmux pass-through sequence */
+    }
+}
+#endif
+
 static void set_norm_xform(int tnr, double *wn, double *vp)
 {
   a[tnr] = (vp[1] - vp[0]) / (wn[1] - wn[0]);
@@ -525,6 +613,7 @@ static void fill_routine(int n, double *px, double *py, int tnr)
   cairo_pattern_t *pattern;
   cairo_surface_t *image;
   cairo_matrix_t pattern_matrix;
+  double scale = 1.0;
 
   WC_to_NDC(px[0], py[0], tnr, x, y);
   seg_xform(&x, &y);
@@ -571,7 +660,7 @@ static void fill_routine(int n, double *px, double *py, int tnr)
               k = (1 << i) & gks_pattern[j];
               if (!(k))
                 {
-                  int p_ind = ((i + 7) % 8) + ((j - 1 + (size - 1)) % size) * 8;
+                  int p_ind = (i + (j - 1) * 8);
                   p->patterns[p_ind * 4 + 3] = (unsigned char)(255 * p->transparency);
                   p->patterns[p_ind * 4 + 2] = (unsigned char)(255 * p->rgb[fl_color][0] * p->transparency);
                   p->patterns[p_ind * 4 + 1] = (unsigned char)(255 * p->rgb[fl_color][1] * p->transparency);
@@ -585,7 +674,8 @@ static void fill_routine(int n, double *px, double *py, int tnr)
       pattern = cairo_pattern_create_for_surface(image);
       cairo_pattern_set_extend(pattern, CAIRO_EXTEND_REPEAT);
       cairo_pattern_set_filter(pattern, CAIRO_FILTER_NEAREST);
-      cairo_matrix_init_scale(&pattern_matrix, 500.0 / min(p->width, p->height), 500.0 / min(p->width, p->height));
+      scale = min(1.0, 500.0 / min(p->width, p->height));
+      cairo_matrix_init_scale(&pattern_matrix, scale, scale);
       cairo_pattern_set_matrix(pattern, &pattern_matrix);
 
       cairo_set_source(p->cr, pattern);
@@ -594,6 +684,7 @@ static void fill_routine(int n, double *px, double *py, int tnr)
   if (fl_inter == GKS_K_INTSTYLE_PATTERN || fl_inter == GKS_K_INTSTYLE_HATCH || fl_inter == GKS_K_INTSTYLE_SOLID)
     {
       cairo_fill(p->cr);
+      if (fl_inter != GKS_K_INTSTYLE_SOLID) cairo_pattern_destroy(pattern);
     }
   else
     {
@@ -816,30 +907,38 @@ static void set_clip_rect(int tnr)
 
   cairo_reset_clip(p->cr);
 
-  if (gkss->clip_tnr != 0 || gkss->clip == GKS_K_CLIP)
+  if (gkss->clip_tnr != 0)
+    tnr = gkss->clip_tnr;
+  else if (gkss->clip == GKS_K_NOCLIP)
+    tnr = 0;
+
+  x = p->rect[tnr][0][0];
+  y = p->rect[tnr][0][1];
+  w = p->rect[tnr][1][0] - p->rect[tnr][0][0];
+  h = p->rect[tnr][1][1] - p->rect[tnr][0][1];
+
+  if (gkss->clip_region == GKS_K_REGION_ELLIPSE)
     {
-      if (gkss->clip_tnr != 0)
+      cairo_save(p->cr);
+      cairo_translate(p->cr, x + 0.5 * w, y + 0.5 * h);
+      cairo_scale(p->cr, 1., h / w);
+      if (gkss->clip_start_angle > 0 || gkss->clip_end_angle < 360)
         {
-          tnr = gkss->clip_tnr;
-        }
-      x = p->rect[tnr][0][0];
-      y = p->rect[tnr][0][1];
-      w = p->rect[tnr][1][0] - p->rect[tnr][0][0];
-      h = p->rect[tnr][1][1] - p->rect[tnr][0][1];
-      if (gkss->clip_region == GKS_K_REGION_ELLIPSE)
-        {
-          cairo_save(p->cr);
-          cairo_translate(p->cr, x + 0.5 * w, y + 0.5 * h);
-          cairo_scale(p->cr, 1., h / w);
-          cairo_arc(p->cr, 0., 0., w * 0.5, 0, 2 * M_PI);
-          cairo_restore(p->cr);
+          cairo_move_to(p->cr, 0, 0);
+          cairo_arc_negative(p->cr, 0, 0, w * 0.5, -gkss->clip_start_angle * M_PI / 180,
+                             -gkss->clip_end_angle * M_PI / 180);
         }
       else
         {
-          cairo_rectangle(p->cr, x, y, w, h);
+          cairo_arc(p->cr, 0., 0., w * 0.5, 0, 2 * M_PI);
         }
-      cairo_clip(p->cr);
+      cairo_restore(p->cr);
     }
+  else
+    {
+      cairo_rectangle(p->cr, x, y, w, h);
+    }
+  cairo_clip(p->cr);
 }
 
 static void set_clipping(int index)
@@ -986,7 +1085,7 @@ static void open_page(void)
 #endif
     }
   else if (p->wtype == 140 || p->wtype == 143 || p->wtype == 144 || p->wtype == 145 || p->wtype == 146 ||
-           p->wtype == 150 || p->wtype == 151)
+           p->wtype == 150 || p->wtype == 151 || p->wtype == 152)
     {
       p->surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, p->width, p->height);
     }
@@ -1663,7 +1762,7 @@ static void write_page(void)
       free(pix);
     }
 #ifndef _WIN32
-  else if (p->wtype == 151)
+  else if (p->wtype == 151 || p->wtype == 152)
     {
       FILE *stream;
       long size, b64_size;
@@ -1703,16 +1802,9 @@ static void write_page(void)
               fprintf(stdout, "\n"); /* only `\n` creates new lines on the screen */
             }
           fprintf(stdout, "\033[%dA", HEIGHT_IN_CELLS); /* Move up `HEIGHT_IN_CELLS` lines */
-          if (p->have_tmux == SINGLE_TMUX_SESSION)
-            {
-              fprintf(stdout, "\033Ptmux;\033"); /* Start a tmux pass-through sequence */
-            }
-          else
-            {
-              fprintf(stdout, "\033Ptmux;\033\033Ptmux;\033\033\033"); /* Start a nested tmux pass-through sequence */
-            }
+          send_tmux_escape_sequence_start();
         }
-      else if (!p->scroll)
+      else if (p->wtype == 151 && !p->scroll)
         {
           if (p->page_counter == 1)
             {
@@ -1723,25 +1815,45 @@ static void write_page(void)
               fprintf(stdout, "\033[H");
             }
         }
-      fprintf(stdout, "\033]1337;File=inline=1;height=" XSTR(HEIGHT_IN_CELLS) ";preserveAspectRatio=0:%s\a",
-              b64_string);
+      if (p->wtype == 151)
+        {
+          /* For a description of the iTerm image protocol, see <https://iterm2.com/documentation-images.html> */
+          long sent_bytes;
+          send_tmux_escaped_escape();
+          fprintf(stdout, "]1337;MultipartFile=inline=1;height=%d;preserveAspectRatio=0\a", HEIGHT_IN_CELLS);
+          for (sent_bytes = 0; sent_bytes < b64_size; sent_bytes += CHUNK_SIZE)
+            {
+              send_tmux_escaped_escape();
+              fprintf(stdout, "]1337;FilePart=%.*s\a", min((int)(b64_size - sent_bytes), CHUNK_SIZE),
+                      b64_string + sent_bytes);
+            }
+          send_tmux_escaped_escape();
+          fprintf(stdout, "]1337;FileEnd\a");
+        }
+      else
+        {
+          /* For a description of the Kitty image protocol, see <https://sw.kovidgoyal.net/kitty/graphics-protocol/> */
+          long sent_bytes;
+          for (sent_bytes = 0; sent_bytes < b64_size; sent_bytes += CHUNK_SIZE)
+            {
+              send_tmux_escaped_escape();
+              fprintf(stdout, "_Gf=100,a=T,q=2,r=%d,m=%d;%.*s", HEIGHT_IN_CELLS,
+                      (sent_bytes < b64_size - CHUNK_SIZE) ? 1 : 0, min((int)(b64_size - sent_bytes), CHUNK_SIZE),
+                      b64_string + sent_bytes);
+              send_tmux_escaped_escape();
+              fprintf(stdout, "\\");
+            }
+        }
       if (p->have_tmux)
         {
-          if (p->have_tmux == SINGLE_TMUX_SESSION)
-            {
-              fprintf(stdout, "\033\\"); /* End a tmux pass-through sequence */
-            }
-          else
-            {
-              fprintf(stdout, "\033\033\\\033\\"); /* End a nested tmux pass-through sequence */
-            }
+          send_tmux_escape_sequence_end();
           /*
            * tmux does not recognize the drawn image and reserves no space for it
            * -> place the cursor at the end of the drawn image
            */
           fprintf(stdout, "\033[%dB", HEIGHT_IN_CELLS); /* Move down `HEIGHT_IN_CELLS` lines */
         }
-      else if (!p->scroll)
+      else if (p->wtype == 151 && !p->scroll)
         {
           fprintf(stdout, "\033[%dH\n", HEIGHT_IN_CELLS);
         }
@@ -2181,49 +2293,6 @@ static void gdp(int n, double *px, double *py, int primid, int nc, int *codes)
     }
 }
 
-#ifndef _WIN32
-static enum tmux_state_t have_tmux(void)
-{
-  const char *term_env_var = gks_getenv("TERM");
-  if (term_env_var == NULL)
-    {
-      return NO_TMUX;
-    }
-
-  if (strncmp(term_env_var, "screen", 6) == 0 || strncmp(term_env_var, "tmux", 4) == 0)
-    {
-      /* Check if the tmux session is running locally, otherwise the server cannot be queried */
-      if (gks_getenv("TMUX") != NULL)
-        {
-          FILE *fp;
-          char client_termname[80];
-
-          /* Inside tmux we can query the tmux server for the outer terminal name */
-          fp = popen("tmux display -p '#{client_termname}'", "r");
-          if (fp == NULL)
-            {
-              /* Reading failed, assume a single tmux session */
-              return SINGLE_TMUX_SESSION;
-            }
-          /* Read the output a line at a time - output it. */
-          if (fgets(client_termname, sizeof(client_termname), fp) == NULL)
-            {
-              /* Reading failed, assume a single tmux session */
-              pclose(fp);
-              return SINGLE_TMUX_SESSION;
-            }
-          pclose(fp);
-          return (strncmp(client_termname, "screen", 6) == 0 || strncmp(client_termname, "tmux", 4) == 0)
-                     ? NESTED_TMUX_SESSION
-                     : SINGLE_TMUX_SESSION;
-        }
-      return SINGLE_TMUX_SESSION;
-    }
-
-  return NO_TMUX;
-}
-#endif
-
 void gks_cairoplugin(int fctid, int dx, int dy, int dimx, int *ia, int lr1, double *r1, int lr2, double *r2, int lc,
                      char *chars, void **ptr)
 {
@@ -2249,7 +2318,7 @@ void gks_cairoplugin(int fctid, int dx, int dy, int dimx, int *ia, int lr1, doub
       p->path = chars;
       p->wtype = ia[2];
 #ifndef _WIN32
-      if (p->wtype == 151)
+      if (p->wtype == 151 || p->wtype == 152)
         {
           p->have_tmux = have_tmux();
         }
@@ -2260,7 +2329,8 @@ void gks_cairoplugin(int fctid, int dx, int dy, int dimx, int *ia, int lr1, doub
 #endif
       p->mem = NULL;
 
-      if (p->wtype == 140 || p->wtype == 144 || p->wtype == 145 || p->wtype == 146 || p->wtype == 151)
+      if (p->wtype == 140 || p->wtype == 144 || p->wtype == 145 || p->wtype == 146 || p->wtype == 151 ||
+          p->wtype == 152)
         {
           p->mw = 0.28575;
           p->mh = 0.19685;
@@ -2269,6 +2339,7 @@ void gks_cairoplugin(int fctid, int dx, int dy, int dimx, int *ia, int lr1, doub
           p->dpi = 600;
           resize(2400, 2400);
           p->nominal_size = 2400 / 500.0;
+          if (gkss->nominal_size > 0) p->nominal_size *= gkss->nominal_size;
         }
       else if (p->wtype == 143)
         {
@@ -2328,14 +2399,8 @@ void gks_cairoplugin(int fctid, int dx, int dy, int dimx, int *ia, int lr1, doub
           p->mw = p->w * 2.54 / 100 / p->dpi;
           p->mh = p->h * 2.54 / 100 / p->dpi;
           resize(width, height);
-          if (gkss->resize_behaviour == GKS_K_RESIZE)
-            {
-              p->nominal_size = min(p->width, p->height) / 500.0;
-            }
-          else
-            {
-              p->nominal_size = 2400 / 500.0;
-            }
+          p->nominal_size = min(p->width, p->height) / 500.0;
+          if (gkss->nominal_size > 0) p->nominal_size *= gkss->nominal_size;
         }
       else if (p->wtype == 150)
         {
@@ -2345,7 +2410,8 @@ void gks_cairoplugin(int fctid, int dx, int dy, int dimx, int *ia, int lr1, doub
           p->h = 420;
           p->dpi = 100;
           resize(400, 400);
-          p->nominal_size = 0.8;
+          p->nominal_size = 400 / 500.0;
+          if (gkss->nominal_size > 0) p->nominal_size *= gkss->nominal_size;
         }
       else
         {
@@ -2356,6 +2422,7 @@ void gks_cairoplugin(int fctid, int dx, int dy, int dimx, int *ia, int lr1, doub
           p->dpi = 100;
           resize(500, 500);
           p->nominal_size = 1;
+          if (gkss->nominal_size > 0) p->nominal_size = gkss->nominal_size;
         }
 
       p->max_points = MAX_POINTS;
@@ -2580,10 +2647,8 @@ void gks_cairoplugin(int fctid, int dx, int dy, int dimx, int *ia, int lr1, doub
             {
               p->width = p->viewport[1] * p->w / p->mw;
               p->height = p->viewport[3] * p->h / p->mh;
-              if (gkss->resize_behaviour == GKS_K_RESIZE)
-                {
-                  p->nominal_size = min(p->width, p->height) / 500.0;
-                }
+              p->nominal_size = min(p->width, p->height) / 500.0;
+              if (gkss->nominal_size > 0) p->nominal_size *= gkss->nominal_size;
             }
           close_page();
           open_page();
@@ -2595,6 +2660,11 @@ void gks_cairoplugin(int fctid, int dx, int dy, int dimx, int *ia, int lr1, doub
       unlock();
       break;
 
+    case 109:
+      p->nominal_size = min(p->width, p->height) / 500.0;
+      if (gkss->nominal_size > 0) p->nominal_size *= gkss->nominal_size;
+      break;
+
     case 203:
       /* set transparency */
       lock();
