@@ -8,6 +8,8 @@
 #include <math.h>
 #include <float.h>
 #include <stack>
+
+#include <QtGlobal>
 #include <QDebug>
 
 #endif
@@ -107,10 +109,12 @@ typedef struct ws_state_list_t
   gks_display_list_t dl;
   QWidget *widget;
   QPixmap *pixmap;
+  QPixmap *bg;
   QPixmap *selection;
   QPainter *painter;
   int state, wtype;
   int device_dpi_x, device_dpi_y;
+  bool has_user_defined_device_pixel_ratio;
   double device_pixel_ratio;
   double mwidth, mheight;
   int width, height;
@@ -131,6 +135,7 @@ typedef struct ws_state_list_t
   int pcolor[PATTERNS];
   bool empty;
   bool prevent_resize_by_dl;
+  bool window_stays_on_top;
   bool interp_was_called;
 
   void (*memory_plugin)(int, int, int, int, int *, int, double *, int, double *, int, char *, void **);
@@ -224,11 +229,10 @@ static void resize_window(void)
       p->height = 2;
       p->mheight = (double)p->height / p->device_dpi_y * 0.0254;
     }
-  if (gkss->resize_behaviour == GKS_K_RESIZE)
-    {
-      p->nominal_size = min(p->width, p->height) / 500.0;
-    }
+  p->nominal_size = min(p->width, p->height) / 500.0;
+  if (gkss->nominal_size > 0) p->nominal_size *= gkss->nominal_size;
 
+#ifndef QT_PLUGIN_USED_AS_PLUGIN_CODE
   if (p->pixmap)
     {
       if (fabs(p->width * p->device_pixel_ratio - p->pixmap->size().width()) > FEPS ||
@@ -242,18 +246,24 @@ static void resize_window(void)
           p->pixmap->setDevicePixelRatio(p->device_pixel_ratio);
 #endif
           p->pixmap->fill(Qt::white);
+          if (p->bg)
+            {
+              delete p->bg;
+              p->bg = new QPixmap(*p->pixmap);
+            }
 
           p->painter = new QPainter(p->pixmap);
           p->painter->setClipRect(0, 0, p->width, p->height);
         }
     }
+#endif
 }
 
 static void set_xform(void)
 {
   double ratio, w, h, x, y;
 
-  ratio = (p->window[1] - p->window[0]) / (p->window[3] - p->window[2]);
+  ratio = (p->window[1] - p->window[0]) / (p->window[3] - p->window[2]) * (1.0 * p->device_dpi_x / p->device_dpi_y);
 
   if (p->width > p->height * ratio)
     {
@@ -275,10 +285,8 @@ static void set_xform(void)
   p->c = h / (p->window[2] - p->window[3]);
   p->d = y + p->window[2] * p->c;
 
-  if (gkss->resize_behaviour == GKS_K_RESIZE)
-    {
-      p->nominal_size = min(p->width, p->height) / 500.0;
-    }
+  p->nominal_size = min(p->width, p->height) / 500.0;
+  if (gkss->nominal_size > 0) p->nominal_size *= gkss->nominal_size;
 }
 
 static void seg_xform(double *x, double *y)
@@ -302,11 +310,26 @@ static void seg_xform_rel(double *x, double *y)
 static void set_clip_rect(int tnr)
 {
   if (gkss->clip_tnr != 0)
-    p->painter->setClipRect(p->rect[gkss->clip_tnr]);
-  else if (gkss->clip == GKS_K_CLIP)
-    p->painter->setClipRect(p->rect[tnr]);
+    tnr = gkss->clip_tnr;
+  else if (gkss->clip == GKS_K_NOCLIP)
+    tnr = 0;
+
+  if (gkss->clip_region == GKS_K_REGION_ELLIPSE && tnr != 0)
+    {
+      if (gkss->clip_start_angle > 0 || gkss->clip_end_angle < 360)
+        {
+          QPainterPath path;
+          path.moveTo(p->rect[tnr].center());
+          path.arcTo(p->rect[tnr].toRect(), gkss->clip_start_angle, gkss->clip_end_angle - gkss->clip_start_angle);
+          p->painter->setClipPath(path);
+        }
+      else
+        p->painter->setClipRegion(QRegion(p->rect[tnr].toRect(), QRegion::Ellipse));
+    }
   else
-    p->painter->setClipRect(p->rect[0]);
+    {
+      p->painter->setClipRect(p->rect[tnr]);
+    }
 }
 
 static void set_color_rep(int color, double red, double green, double blue)
@@ -933,7 +956,7 @@ static void cellarray(double xmin, double xmax, double ymin, double ymax, int dx
 
   if (!true_color)
     {
-      QImage img = QImage(width, height, QImage::Format_RGB32);
+      QImage img = QImage(width, height, QImage::Format_ARGB32);
 #if QT_VERSION >= QT_VERSION_CHECK(5, 1, 0)
       img.setDevicePixelRatio(p->device_pixel_ratio);
 #endif
@@ -1502,6 +1525,10 @@ static void memory_plugin_dl_render(int fctid, int dx, int dy, int dimx, int *ia
           p->viewport[3] = r2[1];
         }
       break;
+    case 109:
+      p->nominal_size = min(p->width, p->height) / 500.0;
+      if (gkss->nominal_size > 0) p->nominal_size *= gkss->nominal_size;
+      break;
     }
   if (p->memory_plugin_initialised)
     {
@@ -1650,6 +1677,11 @@ static void qt_dl_render(int fctid, int dx, int dy, int dimx, int *ia, int lr1,
       init_norm_xform();
       break;
 
+    case 109:
+      p->nominal_size = min(p->width, p->height) / 500.0;
+      if (gkss->nominal_size > 0) p->nominal_size *= gkss->nominal_size;
+      break;
+
     case 203:
       p->transparency = (int)(r1[0] * 255);
       break;
@@ -1702,6 +1734,22 @@ static void qt_dl_render(int fctid, int dx, int dy, int dimx, int *ia, int lr1,
       top->fun_call(top->item_id, top->x_min, top->x_max, top->y_min, top->y_max);
       p->bounding_stack.pop();
       break;
+
+    case SET_BACKGROUND:
+      if (p->pixmap)
+        {
+          if (p->bg) delete p->bg;
+          p->bg = new QPixmap(*p->pixmap);
+        }
+      break;
+
+    case CLEAR_BACKGROUND:
+      if (p->bg)
+        {
+          delete p->bg;
+          p->bg = NULL;
+        }
+      break;
     }
 }
 
@@ -1735,13 +1783,40 @@ static void dl_render_function(int fctid, int dx, int dy, int dimx, int *ia, int
     }
 }
 
+static bool dl_contains_only_background_fctid(const char *s)
+{
+  int sp = 0, *len = NULL, *fctid = NULL;
+  // see `purge` in `dl.c` for preserved fctids (+ `2` for openws)
+  const std::vector<int> leading_ignored_fctids = {2, 48, 54, 55};
+
+  while (true)
+    {
+      RESOLVE(len, int, sizeof(int));
+      if (*len == 0) break;
+      RESOLVE(fctid, int, sizeof(int));
+      if (std::find(leading_ignored_fctids.begin(), leading_ignored_fctids.end(), *fctid) ==
+          leading_ignored_fctids.end())
+        break;
+      sp += *len - 2 * sizeof(int);
+    }
+
+  return *fctid == CLEAR_BACKGROUND || *fctid == SET_BACKGROUND;
+}
+
 static void interp(char *str)
 {
   char *s;
-  int sp = 0, *len;
+  int sp = 0, *len = NULL;
 
   s = str;
 
+  if (p->bg && !dl_contains_only_background_fctid(s))
+    {
+      if (gkss->cntnr != 0) set_clip_rect(0);
+      p->painter->drawPixmap(QPoint(0, 0), *p->bg);
+      if (gkss->cntnr != 0) set_clip_rect(gkss->cntnr);
+    }
+
   RESOLVE(len, int, sizeof(int));
   while (*len)
     {
@@ -1761,7 +1836,7 @@ static void initialize_data()
 {
   int i;
 
-  p->pixmap = p->selection = NULL;
+  p->pixmap = p->bg = p->selection = NULL;
   p->font = new QFont();
 
   p->points = new QPolygonF(MAX_POINTS);
@@ -1781,6 +1856,7 @@ static void initialize_data()
 
   p->memory_plugin_initialised = false;
   p->prevent_resize_by_dl = false;
+  p->window_stays_on_top = false;
   p->interp_was_called = false;
 
   p->window[0] = 0.0;
@@ -1801,56 +1877,25 @@ static void release_data()
   delete p->polygon;
   delete p->points;
   delete p->font;
+#ifndef QT_PLUGIN_USED_AS_PLUGIN_CODE
+  /* The pixmap is only owned if the code is not compiled as plugin. In plugin mode, the pixmap is only a non-owning
+   * pointer to the underlying paint device of the painter object. */
+  if (p->pixmap) delete p->pixmap;
+#endif
+  if (p->bg) delete p->bg;
   delete p;
 }
 
-static int get_paint_device(void)
+static void update_metrics(const QPaintDevice *device)
 {
-  char *env;
-  QPaintDevice *device;
-  bool has_explicit_device_pixel_ratio = false;
-
-  env = (char *)gks_getenv("GKS_CONID");
-  if (!env) env = (char *)gks_getenv("GKSconid");
-
-  if (env != NULL)
-    {
-      bool has_exclamation_mark = strchr(env, '!');
-      bool has_hash_mark = strchr(env, '#');
-      if (has_exclamation_mark && has_hash_mark)
-        {
-          sscanf(env, "%p!%p#%lf", (void **)&p->widget, (void **)&p->painter, &p->device_pixel_ratio);
-          device = p->widget;
-          has_explicit_device_pixel_ratio = true;
-        }
-      else if (has_exclamation_mark)
-        {
-          sscanf(env, "%p!%p", (void **)&p->widget, (void **)&p->painter);
-          device = p->widget;
-        }
-      else if (has_hash_mark)
-        {
-          sscanf(env, "%p#%lf", (void **)&p->painter, &p->device_pixel_ratio);
-          p->widget = NULL;
-          device = p->painter->device();
-          has_explicit_device_pixel_ratio = true;
-        }
-      else
-        {
-          sscanf(env, "%p", (void **)&p->painter);
-          p->widget = NULL;
-          device = p->painter->device();
-        }
-    }
-  else
-    {
-      return 1;
-    }
-
   p->width = device->width();
   p->height = device->height();
-  if (has_explicit_device_pixel_ratio)
+  if (p->has_user_defined_device_pixel_ratio)
     {
+      /* This case was introduced to work around broken HiDPI support in `QQuickPaintedItem`. QML uses phyiscal instead
+       * of logical pixels, but this plugin works with logical pixels. Therefore, convert the physical `width` and
+       * `height` to logical pixels by dividing by `p->device_pixel_ratio` given by the user. Multiply with the
+       * `device_pixel_ratio` set on the paint device to support logical pixels in QML in the future. */
 #if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
       p->width *= device->devicePixelRatioF() / p->device_pixel_ratio;
       p->height *= device->devicePixelRatioF() / p->device_pixel_ratio;
@@ -1876,17 +1921,65 @@ static int get_paint_device(void)
   p->device_dpi_y = device->physicalDpiY();
   p->mwidth = (double)p->width / p->device_dpi_x * 0.0254;
   p->mheight = (double)p->height / p->device_dpi_y * 0.0254;
-  if (gkss->resize_behaviour == GKS_K_RESIZE)
+  p->nominal_size = min(p->width, p->height) / 500.0;
+  if (gkss->nominal_size > 0) p->nominal_size *= gkss->nominal_size;
+}
+
+static int get_paint_device(void)
+{
+  char *env;
+  QPaintDevice *device;
+
+  env = (char *)gks_getenv("GKS_CONID");
+  if (!env) env = (char *)gks_getenv("GKSconid");
+
+  if (env != NULL)
     {
-      p->nominal_size = min(p->width, p->height) / 500.0;
+      bool has_exclamation_mark = strchr(env, '!');
+      bool has_hash_mark = strchr(env, '#');
+      p->has_user_defined_device_pixel_ratio = has_hash_mark;
+      if (has_exclamation_mark && has_hash_mark)
+        {
+          sscanf(env, "%p!%p#%lf", (void **)&p->widget, (void **)&p->painter, &p->device_pixel_ratio);
+          device = p->widget;
+        }
+      else if (has_exclamation_mark)
+        {
+          sscanf(env, "%p!%p", (void **)&p->widget, (void **)&p->painter);
+          device = p->widget;
+        }
+      else if (has_hash_mark)
+        {
+          sscanf(env, "%p#%lf", (void **)&p->painter, &p->device_pixel_ratio);
+          p->widget = NULL;
+          device = p->painter->device();
+        }
+      else
+        {
+          sscanf(env, "%p", (void **)&p->painter);
+          p->widget = NULL;
+          device = p->painter->device();
+        }
+#ifdef QT_PLUGIN_USED_AS_PLUGIN_CODE
+      QPixmap *pixmap = dynamic_cast<QPixmap *>(p->painter->device());
+      if (pixmap != NULL)
+        {
+          p->pixmap = pixmap;
+        }
+#endif
     }
+  else
+    {
+      return 1;
+    }
+
+  update_metrics(device);
 
   return 0;
 }
 
 static void inqdspsize(double *mwidth, double *mheight, int *width, int *height)
 {
-#include <QtGlobal>
 #if QT_VERSION >= 0x050000
   QScreen *screen = QGuiApplication::primaryScreen();
   if (screen)
@@ -1914,6 +2007,7 @@ static void inqdspsize(double *mwidth, double *mheight, int *width, int *height)
 #endif
 }
 
+#ifdef QT_PLUGIN_USED_AS_PLUGIN_CODE
 void QT_PLUGIN_ENTRY_NAME(int fctid, int dx, int dy, int dimx, int *i_arr, int len_f_arr_1, double *f_arr_1,
                           int len_f_arr_2, double *f_arr_2, int len_c_arr, char *c_arr, void **ptr)
 {
@@ -1978,6 +2072,7 @@ void QT_PLUGIN_ENTRY_NAME(int fctid, int dx, int dy, int dimx, int *i_arr, int l
       break;
 
     case 205: /* configure ws */
+      if (p->widget != NULL) update_metrics(p->widget);
       f_arr_1[0] = p->mwidth;
       f_arr_2[0] = p->mheight;
       i_arr[0] = p->width;
@@ -1985,7 +2080,9 @@ void QT_PLUGIN_ENTRY_NAME(int fctid, int dx, int dy, int dimx, int *i_arr, int l
       return;
 
     case 209: /* inq_ws_state */
-      aspect_ratio = (p->window[1] - p->window[0]) / (p->window[3] - p->window[2]);
+      aspect_ratio =
+          (p->window[1] - p->window[0]) / (p->window[3] - p->window[2]) * (1.0 * p->device_dpi_x / p->device_dpi_y);
+      ;
       get_paint_device();
       if (p->width > p->height * aspect_ratio)
         {
@@ -2007,29 +2104,40 @@ void QT_PLUGIN_ENTRY_NAME(int fctid, int dx, int dy, int dimx, int *i_arr, int l
     gks_dl_write_item(&p->dl, fctid, dx, dy, dimx, i_arr, len_f_arr_1, f_arr_1, len_f_arr_2, f_arr_2, len_c_arr, c_arr,
                       gkss);
 }
+#endif
 
 #else
 
-void QT_PLUGIN_ENTRY_NAME(int fctid, int dx, int dy, int dimx, int *ia, int lr1, double *r1, int lr2, double *r2,
-                          int lc, char *chars, void **ptr)
+#ifdef QT_PLUGIN_USED_AS_PLUGIN_CODE
+#define QT_NAME_STRING "Qt"
+
+void QT_PLUGIN_ENTRY_NAME(int fctid, int dx, int dy, int dimx, int *i_arr, int len_f_arr_1, double *f_arr_1,
+                          int len_f_arr_2, double *f_arr_2, int len_c_arr, char *c_arr, void **ptr)
 {
   GKS_UNUSED(dx);
   GKS_UNUSED(dy);
   GKS_UNUSED(dimx);
-  GKS_UNUSED(ia);
-  GKS_UNUSED(lr1);
-  GKS_UNUSED(r1);
-  GKS_UNUSED(lr2);
-  GKS_UNUSED(r2);
-  GKS_UNUSED(lc);
-  GKS_UNUSED(chars);
+  GKS_UNUSED(i_arr);
+  GKS_UNUSED(len_f_arr_1);
+  GKS_UNUSED(f_arr_1);
+  GKS_UNUSED(len_f_arr_2);
+  GKS_UNUSED(f_arr_2);
+  GKS_UNUSED(len_c_arr);
+  GKS_UNUSED(c_arr);
   GKS_UNUSED(ptr);
 
   if (fctid == 2)
     {
       gks_perror(QT_NAME_STRING " support not compiled in");
-      ia[0] = 0;
+      i_arr[0] = 0;
+      f_arr_1[0] = 0;
+      f_arr_2[0] = 0;
+      i_arr[0] = 0;
+      i_arr[1] = 0;
+      if (c_arr != nullptr) c_arr[0] = '\0';
     }
 }
 
 #endif
+
+#endif
